@@ -30,9 +30,33 @@
 
 #include <libgnomevfs/gnome-vfs-cancellation.h>
 #include <libgnomevfs/gnome-vfs-handle.h>
-#include <libgnomevfs/gnome-vfs-process.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
+#ifdef G_OS_WIN32
+#include <io.h>
+
+#undef GNOME_VFS_DATADIR
+extern char *_gnome_vfs_datadir;
+#define GNOME_VFS_DATADIR _gnome_vfs_datadir
+
+#undef GNOME_VFS_LIBDIR
+extern char *_gnome_vfs_libdir;
+#define GNOME_VFS_LIBDIR _gnome_vfs_libdir
+
+#undef GNOME_VFS_PREFIX
+extern char *_gnome_vfs_prefix;
+#define GNOME_VFS_PREFIX _gnome_vfs_prefix
+
+#undef GNOME_VFS_LOCALEDIR
+extern char *_gnome_vfs_localedir;
+#define GNOME_VFS_LOCALEDIR _gnome_vfs_localedir
+
+#undef GNOME_VFS_SYSCONFDIR
+extern char *_gnome_vfs_sysconfdir;
+#define GNOME_VFS_SYSCONFDIR _gnome_vfs_sysconfdir
+
+#endif
+
 G_BEGIN_DECLS
 
 gboolean	 _gnome_vfs_have_ipv6			 (void);
@@ -40,28 +64,12 @@ gboolean	 _gnome_vfs_have_ipv6			 (void);
 gchar   	*_gnome_vfs_canonicalize_pathname         (char *path);
 GnomeVFSResult   gnome_vfs_remove_optional_escapes 	 (char *escaped_uri);
 
-pid_t	 	 gnome_vfs_forkexec 	(const gchar *file_name,
-					 const gchar * const argv[],
-					 GnomeVFSProcessOptions options,
-					 GnomeVFSProcessInitFunc init_func,
-					 gpointer data);
-GnomeVFSProcessRunResult
-	 	gnome_vfs_process_run_cancellable
-					(const gchar *file_name,
-					 const gchar * const argv[],
-					 GnomeVFSProcessOptions options,
-					 GnomeVFSCancellation *cancellation,
-					 guint *exit_value);
-
 GnomeVFSResult	gnome_vfs_create_temp 	(const gchar *prefix,
 					 gchar **name_return,
 					 GnomeVFSHandle **handle_return);
 gboolean	gnome_vfs_atotm		(const gchar *time_string,
 					 time_t *value_return);
 
-GList	       *gnome_vfs_i18n_get_language_list
-					(const gchar *category_name);
-
 GnomeVFSURI    *gnome_vfs_uri_new_private (const gchar *text_uri, 
 					   gboolean allow_unknown_method,
 					   gboolean allow_unsafe_method,
@@ -77,6 +85,9 @@ GnomeVFSResult _gnome_vfs_uri_resolve_all_symlinks_uri (GnomeVFSURI *uri,
 							GnomeVFSURI **result_uri);
 GnomeVFSResult  _gnome_vfs_uri_resolve_all_symlinks (const char *text_uri,
 						     char **resolved_text_uri);
+char *          gnome_vfs_resolve_symlink          (const char *path,
+						    const char *symlink);
+
 
 gboolean  _gnome_vfs_uri_is_in_subdir (GnomeVFSURI *uri, GnomeVFSURI *dir);
 
@@ -88,15 +99,30 @@ gboolean       _gnome_vfs_use_handler_for_scheme          (const char   *scheme)
 gboolean       _gnome_vfs_prepend_terminal_to_vector      (int          *argc,
 							   char       ***argv);
 
-gboolean       _gnome_vfs_set_fd_flags                    (int           fd,
-                                                           int           flags);
-gboolean       _gnome_vfs_clear_fd_flags                  (int           fd,
-                                                           int           flags);
+gboolean       _gnome_vfs_socket_set_blocking             (int       sock_fd,
+		                                           gboolean  blocking);
 
-/* This is private for the moment, due to API freeze */
-GnomeVFSResult gnome_vfs_forget_cache (GnomeVFSHandle *handle,
-				       GnomeVFSFileOffset offset,
-				       GnomeVFSFileSize size);
+int	       _gnome_vfs_pipe				  (int       *fds);
+gboolean       _gnome_vfs_pipe_set_blocking               (int        pipe_fd,
+							   gboolean   blocking);
+
+#ifdef G_OS_WIN32
+void	       _gnome_vfs_map_winsock_error_to_errno	  (void);
+const char    *_gnome_vfs_winsock_strerror		  (int		 error);
+
+#define _GNOME_VFS_SOCKET_IS_ERROR(s) ((s) == SOCKET_ERROR)
+#define _GNOME_VFS_SOCKET_CLOSE(s) closesocket(s)
+#define _GNOME_VFS_SOCKET_READ(s,b,n) recv(s,b,n,0)
+#define _GNOME_VFS_SOCKET_WRITE(s,b,n) send(s,b,n,0)
+
+#else
+
+#define _GNOME_VFS_SOCKET_IS_ERROR(s) ((s) < 0)
+#define _GNOME_VFS_SOCKET_CLOSE(s) close(s)
+#define _GNOME_VFS_SOCKET_READ(s,b,n) read(s,b,n)
+#define _GNOME_VFS_SOCKET_WRITE(s,b,n) write(s,b,n)
+
+#endif
 
 G_END_DECLS
 
