@@ -26,7 +26,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "xdgmimeglob.h"
@@ -52,6 +52,8 @@ struct XdgGlobHashNode
 {
   xdg_unichar_t character;
   const char *mime_type;
+  int weight;
+  int case_sensitive;
   XdgGlobHashNode *next;
   XdgGlobHashNode *child;
 };
@@ -59,6 +61,8 @@ struct XdgGlobList
 {
   const char *data;
   const char *mime_type;
+  int weight;
+  int case_sensitive;
   XdgGlobList *next;
 };
 
@@ -107,14 +111,28 @@ _xdg_glob_list_free (XdgGlobList *glob_list)
 static XdgGlobList *
 _xdg_glob_list_append (XdgGlobList *glob_list,
 		       void        *data,
-		       const char  *mime_type)
+		       const char  *mime_type,
+		       int          weight,
+		       int          case_sensitive)
 {
   XdgGlobList *new_element;
   XdgGlobList *tmp_element;
 
+  tmp_element = glob_list;
+  while (tmp_element != NULL)
+    {
+      if (strcmp (tmp_element->data, data) == 0 &&
+	  strcmp (tmp_element->mime_type, mime_type) == 0)
+	return glob_list;
+
+      tmp_element = tmp_element->next;
+    }
+
   new_element = _xdg_glob_list_new ();
   new_element->data = data;
   new_element->mime_type = mime_type;
+  new_element->weight = weight;
+  new_element->case_sensitive = case_sensitive;
   if (glob_list == NULL)
     return new_element;
 
@@ -127,23 +145,6 @@ _xdg_glob_list_append (XdgGlobList *glob_list,
   return glob_list;
 }
 
-#if 0
-static XdgGlobList *
-_xdg_glob_list_prepend (XdgGlobList *glob_list,
-			void        *data,
-			const char  *mime_type)
-{
-  XdgGlobList *new_element;
-
-  new_element = _xdg_glob_list_new ();
-  new_element->data = data;
-  new_element->next = glob_list;
-  new_element->mime_type = mime_type;
-
-  return new_element;
-}
-#endif
-
 /* XdgGlobHashNode
  */
 
@@ -167,7 +168,7 @@ _xdg_glob_hash_node_dump (XdgGlobHashNode *glob_hash_node,
 
   printf ("%c", (char)glob_hash_node->character);
   if (glob_hash_node->mime_type)
-    printf (" - %s\n", glob_hash_node->mime_type);
+    printf (" - %s %d\n", glob_hash_node->mime_type, glob_hash_node->weight);
   else
     printf ("\n");
   if (glob_hash_node->child)
@@ -177,14 +178,16 @@ _xdg_glob_hash_node_dump (XdgGlobHashNode *glob_hash_node,
 }
 
 static XdgGlobHashNode *
-_xdg_glob_hash_insert_text (XdgGlobHashNode *glob_hash_node,
-			    const char      *text,
-			    const char      *mime_type)
+_xdg_glob_hash_insert_ucs4 (XdgGlobHashNode *glob_hash_node,
+			    xdg_unichar_t   *text,
+			    const char      *mime_type,
+			    int              weight,
+			    int              case_sensitive)
 {
   XdgGlobHashNode *node;
   xdg_unichar_t character;
 
-  character = _xdg_utf8_to_ucs4 (text);
+  character = text[0];
 
   if ((glob_hash_node == NULL) ||
       (character < glob_hash_node->character))
@@ -238,18 +241,18 @@ _xdg_glob_hash_insert_text (XdgGlobHashNode *glob_hash_node,
 	}
     }
 
-  text = _xdg_utf8_next_char (text);
-  if (*text == '\000')
+  text++;
+  if (*text == 0)
     {
       if (node->mime_type)
 	{
-	  if (strcmp (node->mime_type, mime_type))
+	  if (strcmp (node->mime_type, mime_type) != 0)
 	    {
 	      XdgGlobHashNode *child;
 	      int found_node = FALSE;
-	      
+
 	      child = node->child;
-	      while (child && child->character == '\0')
+	      while (child && child->character == 0)
 		{
 		  if (strcmp (child->mime_type, mime_type) == 0)
 		    {
@@ -262,8 +265,10 @@ _xdg_glob_hash_insert_text (XdgGlobHashNode *glob_hash_node,
 	      if (!found_node)
 		{
 		  child = _xdg_glob_hash_node_new ();
-		  child->character = '\000';
-		  child->mime_type = mime_type;
+		  child->character = 0;
+		  child->mime_type = strdup (mime_type);
+		  child->weight = weight;
+		  child->case_sensitive = case_sensitive;
 		  child->child = NULL;
 		  child->next = node->child;
 		  node->child = child;
@@ -272,21 +277,48 @@ _xdg_glob_hash_insert_text (XdgGlobHashNode *glob_hash_node,
 	}
       else
 	{
-	  node->mime_type = mime_type;
+	  node->mime_type = strdup (mime_type);
+	  node->weight = weight;
+	  node->case_sensitive = case_sensitive;
 	}
     }
   else
     {
-      node->child = _xdg_glob_hash_insert_text (node->child, text, mime_type);
+      node->child = _xdg_glob_hash_insert_ucs4 (node->child, text, mime_type, weight, case_sensitive);
     }
   return glob_hash_node;
 }
 
+/* glob must be valid UTF-8 */
+static XdgGlobHashNode *
+_xdg_glob_hash_insert_text (XdgGlobHashNode *glob_hash_node,
+			    const char      *text,
+			    const char      *mime_type,
+			    int              weight,
+			    int              case_sensitive)
+{
+  XdgGlobHashNode *node;
+  xdg_unichar_t *unitext;
+  int len;
+
+  unitext = _xdg_convert_to_ucs4 (text, &len);
+  _xdg_reverse_ucs4 (unitext, len);
+  node = _xdg_glob_hash_insert_ucs4 (glob_hash_node, unitext, mime_type, weight, case_sensitive);
+  free (unitext);
+  return node;
+}
+
+typedef struct {
+  const char *mime;
+  int weight;
+} MimeWeight;
+
 static int
 _xdg_glob_hash_node_lookup_file_name (XdgGlobHashNode *glob_hash_node,
 				      const char      *file_name,
-				      int              ignore_case,
-				      const char      *mime_types[],
+				      int              len,
+				      int              case_sensitive_check,
+				      MimeWeight       mime_types[],
 				      int              n_mime_types)
 {
   int n;
@@ -296,36 +328,47 @@ _xdg_glob_hash_node_lookup_file_name (XdgGlobHashNode *glob_hash_node,
   if (glob_hash_node == NULL)
     return 0;
 
-  character = _xdg_utf8_to_ucs4 (file_name);
-  if (ignore_case)
-    character = _xdg_ucs4_to_lower(character);
+  character = file_name[len - 1];
 
   for (node = glob_hash_node; node && character >= node->character; node = node->next)
     {
       if (character == node->character)
-	{
-	  file_name = _xdg_utf8_next_char (file_name);
-	  if (*file_name == '\000')
-	    {
-	      n = 0;
-              if (node->mime_type)
-	        mime_types[n++] = node->mime_type;
-	      node = node->child;
-	      while (n < n_mime_types && node && node->character == 0)
-		{
-                  if (node->mime_type)
-		    mime_types[n++] = node->mime_type;
-		  node = node->next;
-		}
-	    }
-	  else
+        {
+          len--;
+          n = 0;
+          if (len > 0) 
 	    {
 	      n = _xdg_glob_hash_node_lookup_file_name (node->child,
 							file_name,
-							ignore_case,
+							len,
+							case_sensitive_check,
 							mime_types,
 							n_mime_types);
 	    }
+	  if (n == 0)
+	    {
+              if (node->mime_type &&
+		  (case_sensitive_check ||
+		   !node->case_sensitive))
+                {
+	          mime_types[n].mime = node->mime_type;
+		  mime_types[n].weight = node->weight;
+		  n++; 
+                }
+	      node = node->child;
+	      while (n < n_mime_types && node && node->character == 0)
+		{
+                  if (node->mime_type &&
+		      (case_sensitive_check ||
+		       !node->case_sensitive))
+		    {
+		      mime_types[n].mime = node->mime_type;
+		      mime_types[n].weight = node->weight;
+		      n++;
+		    }
+		  node = node->next;
+		}
+	    }
 	  return n;
 	}
     }
@@ -333,6 +376,30 @@ _xdg_glob_hash_node_lookup_file_name (XdgGlobHashNode *glob_hash_node,
   return 0;
 }
 
+static int compare_mime_weight (const void *a, const void *b)
+{
+  const MimeWeight *aa = (const MimeWeight *)a;
+  const MimeWeight *bb = (const MimeWeight *)b;
+
+  return aa->weight - bb->weight;
+}
+
+#define ISUPPER(c)		((c) >= 'A' && (c) <= 'Z')
+static char *
+ascii_tolower (const char *str)
+{
+  char *p, *lower;
+
+  lower = strdup (str);
+  p = lower;
+  while (*p != 0)
+    {
+      char c = *p;
+      *p++ = ISUPPER (c) ? c - 'A' + 'a' : c;
+    }
+  return lower;
+}
+
 int
 _xdg_glob_hash_lookup_file_name (XdgGlobHash *glob_hash,
 				 const char  *file_name,
@@ -340,55 +407,71 @@ _xdg_glob_hash_lookup_file_name (XdgGlobHash *glob_hash,
 				 int          n_mime_types)
 {
   XdgGlobList *list;
-  const char *ptr;
-  char stopchars[128];
   int i, n;
-  XdgGlobHashNode *node;
+  MimeWeight mimes[10];
+  int n_mimes = 10;
+  int len;
+  char *lower_case;
+  int try_lower_case;
 
   /* First, check the literals */
 
   assert (file_name != NULL && n_mime_types > 0);
 
+  n = 0;
+
+  lower_case = ascii_tolower (file_name);
+
   for (list = glob_hash->literal_list; list; list = list->next)
     {
       if (strcmp ((const char *)list->data, file_name) == 0)
 	{
 	  mime_types[0] = list->mime_type;
+	  free (lower_case);
 	  return 1;
 	}
     }
 
-  i = 0;
-  for (node = glob_hash->simple_node; node; node = node->next)
+  for (list = glob_hash->literal_list; list; list = list->next)
     {
-      if (node->character < 128)
- 	stopchars[i++] = (char)node->character;
-    }
-  stopchars[i] = '\0';
- 
-  ptr = strpbrk (file_name, stopchars);
-  while (ptr)
-    {
-      n = _xdg_glob_hash_node_lookup_file_name (glob_hash->simple_node, ptr, FALSE,
-						mime_types, n_mime_types);
-      if (n > 0)
-	return n;
-      
-      n = _xdg_glob_hash_node_lookup_file_name (glob_hash->simple_node, ptr, TRUE,
-						mime_types, n_mime_types);
-      if (n > 0)
-	return n;
-      
-      ptr = strpbrk (ptr + 1, stopchars);
+      if (!list->case_sensitive &&
+	  strcmp ((const char *)list->data, lower_case) == 0)
+	{
+	  mime_types[0] = list->mime_type;
+	  free (lower_case);
+	  return 1;
+	}
     }
 
-  /* FIXME: Not UTF-8 safe */
-  n = 0;
-  for (list = glob_hash->full_list; list && n < n_mime_types; list = list->next)
+
+  len = strlen (file_name);
+  n = _xdg_glob_hash_node_lookup_file_name (glob_hash->simple_node, lower_case, len, FALSE,
+					    mimes, n_mimes);
+  if (n == 0)
+    n = _xdg_glob_hash_node_lookup_file_name (glob_hash->simple_node, file_name, len, TRUE,
+					      mimes, n_mimes);
+
+  if (n == 0)
     {
-      if (fnmatch ((const char *)list->data, file_name, 0) == 0)
-	mime_types[n++] = list->mime_type;
+      for (list = glob_hash->full_list; list && n < n_mime_types; list = list->next)
+        {
+          if (fnmatch ((const char *)list->data, file_name, 0) == 0)
+	    {
+	      mimes[n].mime = list->mime_type;
+	      mimes[n].weight = list->weight;
+	      n++;
+	    }
+        }
     }
+  free (lower_case);
+
+  qsort (mimes, n, sizeof (MimeWeight), compare_mime_weight);
+
+  if (n_mime_types < n)
+    n = n_mime_types;
+
+  for (i = 0; i < n; i++)
+    mime_types[i] = mimes[i].mime;
 
   return n;
 }
@@ -442,7 +525,7 @@ _xdg_glob_determine_type (const char *glob)
 
   ptr = glob;
 
-  while (*ptr != '\000')
+  while (*ptr != '\0')
     {
       if (*ptr == '*' && first_char)
 	maybe_in_simple_glob = TRUE;
@@ -462,7 +545,9 @@ _xdg_glob_determine_type (const char *glob)
 void
 _xdg_glob_hash_append_glob (XdgGlobHash *glob_hash,
 			    const char  *glob,
-			    const char  *mime_type)
+			    const char  *mime_type,
+			    int          weight,
+			    int          case_sensitive)
 {
   XdgGlobType type;
 
@@ -474,13 +559,13 @@ _xdg_glob_hash_append_glob (XdgGlobHash *glob_hash,
   switch (type)
     {
     case XDG_GLOB_LITERAL:
-      glob_hash->literal_list = _xdg_glob_list_append (glob_hash->literal_list, strdup (glob), strdup (mime_type));
+      glob_hash->literal_list = _xdg_glob_list_append (glob_hash->literal_list, strdup (glob), strdup (mime_type), weight, case_sensitive);
       break;
     case XDG_GLOB_SIMPLE:
-      glob_hash->simple_node = _xdg_glob_hash_insert_text (glob_hash->simple_node, glob + 1, strdup (mime_type));
+      glob_hash->simple_node = _xdg_glob_hash_insert_text (glob_hash->simple_node, glob + 1, mime_type, weight, case_sensitive);
       break;
     case XDG_GLOB_FULL:
-      glob_hash->full_list = _xdg_glob_list_append (glob_hash->full_list, strdup (glob), strdup (mime_type));
+      glob_hash->full_list = _xdg_glob_list_append (glob_hash->full_list, strdup (glob), strdup (mime_type), weight, case_sensitive);
       break;
     }
 }
@@ -490,37 +575,46 @@ _xdg_glob_hash_dump (XdgGlobHash *glob_hash)
 {
   XdgGlobList *list;
   printf ("LITERAL STRINGS\n");
-  if (glob_hash->literal_list == NULL)
+  if (!glob_hash || glob_hash->literal_list == NULL)
     {
       printf ("    None\n");
     }
   else
     {
       for (list = glob_hash->literal_list; list; list = list->next)
-	printf ("    %s - %s\n", (char *)list->data, list->mime_type);
+	printf ("    %s - %s %d\n", (char *)list->data, list->mime_type, list->weight);
     }
   printf ("\nSIMPLE GLOBS\n");
-  _xdg_glob_hash_node_dump (glob_hash->simple_node, 4);
+  if (!glob_hash || glob_hash->simple_node == NULL)
+    {
+      printf ("    None\n");
+    }
+  else
+    {
+      _xdg_glob_hash_node_dump (glob_hash->simple_node, 4);
+    }
 
   printf ("\nFULL GLOBS\n");
-  if (glob_hash->full_list == NULL)
+  if (!glob_hash || glob_hash->full_list == NULL)
     {
       printf ("    None\n");
     }
   else
     {
       for (list = glob_hash->full_list; list; list = list->next)
-	printf ("    %s - %s\n", (char *)list->data, list->mime_type);
+	printf ("    %s - %s %d\n", (char *)list->data, list->mime_type, list->weight);
     }
 }
 
 
 void
 _xdg_mime_glob_read_from_file (XdgGlobHash *glob_hash,
-			       const char  *file_name)
+			       const char  *file_name,
+			       int          version_two)
 {
   FILE *glob_file;
   char line[255];
+  char *p;
 
   glob_file = fopen (file_name, "r");
 
@@ -532,15 +626,66 @@ _xdg_mime_glob_read_from_file (XdgGlobHash *glob_hash,
   while (fgets (line, 255, glob_file) != NULL)
     {
       char *colon;
-      if (line[0] == '#')
+      char *mimetype, *glob, *end;
+      int weight;
+      int case_sensitive;
+
+      if (line[0] == '#' || line[0] == 0)
 	continue;
 
-      colon = strchr (line, ':');
+      end = line + strlen(line) - 1;
+      if (*end == '\n')
+	*end = 0;
+
+      p = line;
+      if (version_two)
+	{
+	  colon = strchr (p, ':');
+	  if (colon == NULL)
+	    continue;
+	  *colon = 0;
+          weight = atoi (p);
+	  p = colon + 1;
+	}
+      else
+	weight = 50;
+
+      colon = strchr (p, ':');
       if (colon == NULL)
 	continue;
-      *(colon++) = '\000';
-      colon[strlen (colon) -1] = '\000';
-      _xdg_glob_hash_append_glob (glob_hash, colon, line);
+      *colon = 0;
+
+      mimetype = p;
+      p = colon + 1;
+      glob = p;
+      case_sensitive = FALSE;
+
+      colon = strchr (p, ':');
+      if (version_two && colon != NULL)
+	{
+	  char *flag;
+
+	  /* We got flags */
+	  *colon = 0;
+	  p = colon + 1;
+
+	  /* Flags end at next colon */
+	  colon = strchr (p, ':');
+	  if (colon != NULL)
+	    *colon = 0;
+
+	  flag = strstr (p, "cs");
+	  if (flag != NULL &&
+	      /* Start or after comma */
+	      (flag == p ||
+	       flag[-1] == ',') &&
+	      /* ends with comma or end of string */
+	      (flag[2] == 0 ||
+	       flag[2] == ','))
+	    case_sensitive = TRUE;
+	}
+
+      _xdg_glob_hash_append_glob (glob_hash, glob, mimetype, weight, case_sensitive);
     }
 
   fclose (glob_file);
